"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
// Given a key, the configuration object (with overrides already applied) and
// the end result object, compute the current value for the given key and write
// that value to the result object. Optionally track the path values are
// computed along in cases where it's possible to define circular dependencies.
function compute(key, values, result, invocations, path = []) {
    const value = values[key];
    // In essence, this is "exhaustively" checking for each type of attribute that
    // can be defined for a given key. Unfortunately it's not truly exhaustive,
    // but would be great to update this to do true exhaustive type checking.
    if (utils_1.isDerivedFunction(value)) {
        result[key] = value(result, values, invocations, path);
    }
    else if (utils_1.isFactoryFunction(value)) {
        result[key] = value();
    }
    else if (utils_1.isAttributeFunction(value)) {
        result[key] = value(invocations);
    }
    else {
        result[key] = value;
    }
}
exports.compute = compute;
